CREATE TABLE TEKLA_INFO
(
  ID DOM_KEY NOT NULL,
  NAME DOM_CHAR_100,
  LAST_SYNC Bigint,
  CONSTRAINT PK_TEKLA_INFO PRIMARY KEY (ID)
);

CREATE TABLE TEKLA_INFO_SYNC
(
  ID Bigint NOT NULL,
  SYNCTIP DOM_KEY_SMALL,
  SKLAD DOM_KEY,
  ID_TO_SYNC DOM_KEY NOT NULL,
  CONSTRAINT PK_TEKLA_INFO_SYNC PRIMARY KEY (ID)
);


CREATE GLOBAL TEMPORARY TABLE TEKLA_SMETKI_INFO
(
  ID Bigint NOT NULL,
  KLI_ID Integer,
  USER_ID Smallint NOT NULL,
  DATETIME Timestamp NOT NULL,
  SUMA Double precision,
  SKLAD_ID Smallint NOT NULL,
  OBJ_ID Smallint,
  IS_F Smallint,
  IP Integer,
  CONSTRAINT PK_TEKLA_SMETKI_INFO PRIMARY KEY (ID)
)
ON COMMIT PRESERVE ROWS;

CREATE GLOBAL TEMPORARY TABLE TEKLA_SMETKI_I_EL
(
  SMETKA_ID Bigint NOT NULL,
  SIMPLE_ID Integer NOT NULL,
  KOL Double precision,
  PRICE_PL Double precision,
  SUMA Double precision,
  PRICE Double precision,
  CONSTRAINT PK_TEKLA_SMETKI_I_EL PRIMARY KEY (SMETKA_ID,SIMPLE_ID)
)
ON COMMIT PRESERVE ROWS;

CREATE GENERATOR GEN_TEKLA_ID;

CREATE GENERATOR GEN_TEKLA_SYNCS;


SET TERM ^ ;
CREATE TRIGGER TEKLA_INFO_AU100 FOR TEKLA_INFO ACTIVE
AFTER UPDATE POSITION 100
AS 
BEGIN 
	DELETE FROM TEKLA_INFO_SYNC TIS WHERE TIS.ID <= (SELECT MIN(LAST_SYNC) FROM TEKLA_INFO TI);
    IF(EXISTS (SELECT * FROM TEKLA_INFO_SYNC TIS WHERE TIS.ID>NEW.LAST_SYNC)) THEN POST_EVENT 'RESYNC_TEKLA' || NEW.ID;
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER TEKLA_INFO_BI0 FOR TEKLA_INFO ACTIVE
BEFORE INSERT POSITION 0
AS 
BEGIN 
	IF(NEW.ID IS NULL) THEN NEW.ID = NEXT VALUE FOR GEN_TEKLA_ID;
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER TEKLA_INFO_SYNC_AI10 FOR TEKLA_INFO_SYNC ACTIVE
AFTER INSERT POSITION 10
AS 
BEGIN 
	POST_EVENT 'SYNC_TEKLA' || coalesce(NEW.SKLAD, ''); 
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER TEKLA_INFO_SYNC_BI0 FOR TEKLA_INFO_SYNC ACTIVE
BEFORE INSERT POSITION 0
AS 
BEGIN 
	IF(NEW.ID IS NULL) THEN NEW.ID = NEXT VALUE FOR GEN_TEKLA_SYNCS;
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER BARCODE_AD100 FOR BARCODE ACTIVE
AFTER DELETE POSITION 100
AS 
BEGIN 
	INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(16,OLD.ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER BARCODE_AIU100 FOR BARCODE ACTIVE
AFTER INSERT OR UPDATE POSITION 100
AS 
BEGIN 
	INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(6,NEW.ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER CENOVA_LISTA_AD100 FOR CENOVA_LISTA ACTIVE
AFTER DELETE POSITION 100
AS 
BEGIN 
	INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(14,OLD.OPR_ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER CENOVA_LISTA_AIU100 FOR CENOVA_LISTA ACTIVE
AFTER INSERT OR UPDATE POSITION 100
AS 
BEGIN 
IF(NEW.ACTIVNA>0) THEN INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(4,NEW.OPR_ID);
ELSE INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(14,NEW.OPR_ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER CENOVA_LISTA_EL_AD100 FOR CENOVA_LISTA_EL ACTIVE
AFTER DELETE POSITION 100
AS 
BEGIN 
	INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(15,OLD.ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER CENOVA_LISTA_EL_AIU100 FOR CENOVA_LISTA_EL ACTIVE
AFTER INSERT OR UPDATE POSITION 100
AS 
BEGIN 
	INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(5,NEW.ID);

END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER CENOVA_LISTA_KLI_AD100 FOR CENOVA_LISTA_KLI ACTIVE
AFTER DELETE POSITION 100
AS 
BEGIN 
	INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(4,OLD.OPR_ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER CENOVA_LISTA_KLI_AIU100 FOR CENOVA_LISTA_KLI ACTIVE
AFTER INSERT OR UPDATE POSITION 100
AS 
BEGIN 
	INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(4,NEW.OPR_ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER CENOVA_LISTA_SKLAD_AD100 FOR CENOVA_LISTA_SKLAD ACTIVE
AFTER INSERT OR UPDATE POSITION 100
AS 
BEGIN 
        IF(NOT (COALESCE(OLD.SKLAD_ID,0)=NEW.SKLAD_ID)) then 
            begin
        INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC,SKLAD) VALUES(4,NEW.OPR_ID,NEW.SKLAD_ID);
        IF(EXISTS(SELECT * FROM CENOVA_LISTA_SKLAD WHERE OPR_ID=OLD.OPR_ID)) THEN INSERT INTO TEKLA_INFO_SYNC(SKLAD,SYNCTIP,ID_TO_SYNC) VALUES(OLD.SKLAD_ID,14,OLD.OPR_ID);
            end
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER CENOVA_LISTA_SKLAD_AIU0 FOR CENOVA_LISTA_SKLAD ACTIVE
AFTER INSERT OR UPDATE POSITION 100
AS 
BEGIN 
        IF((NOT(OLD.ID IS NULL)) AND (NOT(OLD.ID=NEW.ID)) AND EXISTS(SELECT * FROM CENOVA_LISTA_SKLAD WHERE OPR_ID=OLD.OPR_ID))THEN INSERT INTO TEKLA_INFO_SYNC(SKLAD,SYNCTIP,ID_TO_SYNC) VALUES(OLD.SKLAD_ID,14,OLD.OPR_ID);
INSERT INTO TEKLA_INFO_SYNC(SKLAD,SYNCTIP,ID_TO_SYNC) VALUES(NEW.SKLAD_ID,4,NEW.OPR_ID);
    
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER KLI_AD0 FOR KLI ACTIVE
AFTER DELETE POSITION 0
AS 
BEGIN 
	IF(coalesce(OLD.KLIENT,0)>0) THEN INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(17,OLD.ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER KLI_AD10 FOR KLI ACTIVE
AFTER DELETE POSITION 100
AS 
BEGIN 
	INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(17,OLD.ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER KLI_AIU0 FOR KLI ACTIVE
AFTER INSERT OR UPDATE POSITION 100
AS 
BEGIN 
	INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(7,NEW.ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER ME_AD100 FOR ME ACTIVE
AFTER DELETE POSITION 100
AS 
BEGIN 
        INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(11,OLD.ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER ME_AIU100 FOR ME ACTIVE
AFTER INSERT OR UPDATE POSITION 100
AS 
BEGIN 
        INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(1,NEW.ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER NAL_AD100 FOR NAL ACTIVE
AFTER DELETE POSITION 100
AS 
BEGIN 
	INSERT INTO TEKLA_INFO_SYNC(SKLAD,SYNCTIP,ID_TO_SYNC) VALUES (OLD.SKLAD_ID,13,OLD.SIMPLE_ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER NAL_AIU100 FOR NAL ACTIVE
AFTER INSERT OR UPDATE POSITION 100
AS 
BEGIN 
	INSERT INTO TEKLA_INFO_SYNC(SKLAD,SYNCTIP,ID_TO_SYNC) VALUES (NEW.SKLAD_ID,3,NEW.SIMPLE_ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER OPR_ANUL_AIU100 FOR OPR_ANUL ACTIVE
AFTER INSERT OR UPDATE POSITION 100
AS 
BEGIN 
	IF(EXISTS (SELECT * FROM OPR WHERE OPR_TIP=25 AND ID=NEW.AN_OPR_ID)) THEN INSERT INTO TEKLA_INFO_SYNC(ID_TO_SYNC,SYNCTIP) VALUES(NEW.AN_OPR_ID,14);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER SIMPLE_AD100 FOR SIMPLE ACTIVE
AFTER DELETE POSITION 100
AS 
BEGIN 
        INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(12,OLD.ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER SIMPLE_AIU100 FOR SIMPLE ACTIVE
AFTER INSERT OR UPDATE POSITION 100
AS 
BEGIN 
      if((not (NEW.ME_ID IS NULL)) AND NEW."ACTIVE">0) THEN  INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(2,NEW.ID);
      if(NEW."ACTIVE"=0) THEN  INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(12,NEW.ID);
        
END^
SET TERM ; ^



SET TERM ^ ;
CREATE TRIGGER USER_AD100 FOR "USER" ACTIVE
AFTER DELETE POSITION 100
AS 
BEGIN 
	INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(19,OLD.ID);
END^
SET TERM ; ^

SET TERM ^ ;
CREATE TRIGGER USER_AIU100 FOR "USER" ACTIVE
AFTER INSERT OR UPDATE POSITION 100
AS
 
BEGIN 
	IF(NEW.IS_ACTIVE>0) THEN INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(9,NEW.ID);
    ELSE INSERT INTO TEKLA_INFO_SYNC(SYNCTIP,ID_TO_SYNC) VALUES(19,NEW.ID);
END^
SET TERM ; ^




SET TERM ^ ;
CREATE PROCEDURE TEKLA_COMMIT_ALL_SMETKI
AS
DECLARE VARIABLE S_ID BIGINT; 
DECLARE VARIABLE S_KLI_ID INT;
DECLARE VARIABLE S_USER_ID SMALLINT;
DECLARE VARIABLE S_DATETIME TIMESTAMP;
DECLARE VARIABLE S_SUMA DOUBLE precision;
DECLARE VARIABLE S_SKLAD_ID SMALLINT;
DECLARE VARIABLE S_OBJ_ID SMALLINT;
DECLARE VARIABLE S_IP INT;
DECLARE VARIABLE S_IS_F SMALLINT;
DECLARE VARIABLE S_EL_SIMPLE_ID INT;
DECLARE VARIABLE S_EL_KOL DOUBLE precision;
DECLARE VARIABLE S_EL_PRICE_PL DOUBLE precision;
DECLARE VARIABLE S_EL_PRICE DOUBLE PRECISION;
DECLARE VARIABLE S_EL_SUMA DOUBLE precision;


DECLARE VARIABLE O_ID INT;
DECLARE VARIABLE OU_ID INT;
DECLARE VARIABLE OU_EL_ID INT;
DECLARE VARIABLE SUMA_SR DOUBLE precision;
DECLARE VARIABLE SUMA_FR DOUBLE PRECISION;

BEGIN
FOR SELECT 
    TSI.ID,
    TSI.KLI_ID,
    TSI.USER_ID,
    TSI.SKLAD_ID,
    TSI.OBJ_ID,
    TSI.SUMA,
    TSI.DATETIME,
    TSI.IS_F,
    TSI.IP
FROM TEKLA_SMETKI_INFO TSI
INTO :S_ID,:S_KLI_ID,:S_USER_ID,:S_SKLAD_ID,:S_OBJ_ID,:S_SUMA,:S_DATETIME,:S_IS_F,:S_IP
DO 
begin
    EXECUTE PROCEDURE ADD_OPR(2,CAST(:S_DATETIME AS DATE),:S_IP,:S_USER_ID,:S_SKLAD_ID,NULLIF(:S_KLI_ID,0),:S_SUMA) RETURNING_VALUES :O_ID;
        
    for SELECT
        TSIE.SIMPLE_ID,
        TSIE.KOL,
        TSIE.SUMA,
        TSIE.PRICE_PL,
        TSIE.PRICE
    FROM TEKLA_SMETKI_I_EL TSIE
    WHERE TSIE.SMETKA_ID=:S_ID 
    INTO :S_EL_SIMPLE_ID,:S_EL_KOL,:S_EL_SUMA,:S_EL_PRICE_PL,:S_EL_PRICE
    DO
        begin
        EXECUTE PROCEDURE GET_SUMA_SR(:S_EL_SIMPLE_ID,:S_SKLAD_ID,:S_EL_KOL) RETURNING_VALUES :SUMA_SR;
        EXECUTE PROCEDURE ADD_OUT_EL(:O_ID,:S_EL_SIMPLE_ID,:S_EL_KOL,:S_EL_SUMA,COALESCE(SUMA_SR,0),0,'0',null) RETURNING_VALUES :OU_EL_ID;
        INSERT INTO PROD_CENI_EL(OUT_EL_ID,CENA,CENA_CL) VALUES(:OU_EL_ID,:S_EL_PRICE,:S_EL_PRICE_PL);
        EXECUTE PROCEDURE PARTIDA_OUT(:S_EL_KOL,:S_EL_SIMPLE_ID,:S_SKLAD_ID,:O_ID,:OU_EL_ID,0,null,null,1) RETURNING_VALUES :S_EL_KOL,:S_EL_SUMA;--polzvame tezi, ve4e ne ni trqvat
        update opr set DATETIME = :S_DATETIME WHERE ID = :O_ID;
       
    END
    SELECT SUM (SUMA_SR), SUM (SUMA_FR) FROM OUT_EL WHERE OPR_ID = :O_ID INTO :SUMA_SR,:SUMA_FR;
    EXECUTE PROCEDURE ADD_OUT(:O_ID,0,0,SUMA_SR,SUMA_FR,:S_IS_F,null,:S_OBJ_ID,0,null) RETURNING_VALUES :OU_ID;
    INSERT INTO OPR_SUMI(OPR_ID,SUMA_FR,SUMA_SR) VALUES(:O_ID,:SUMA_FR,:SUMA_SR);
    DELETE FROM TEKLA_SMETKI_INFO WHERE ID = :S_ID;
end
END^
SET TERM ; ^

GRANT EXECUTE
 ON PROCEDURE TEKLA_COMMIT_ALL_SMETKI TO  SYSDBA;

